// VUMetter.cpp: implementation of the CVUMetter class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MusicTime.h"
#include "VUMetter.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CVUMetter::CVUMetter()
{
	pw=NULL;
	oldDept=0;
	oy=ox=sx=0;
	PI=atan(1)*4;
	kPI=PI/180;
	LvL=0;
	for(int i=0;i<256;zarc[i++]=0);
	SpectScale=50.0f;
	PEAK=FALSE;
}

CVUMetter::~CVUMetter()
{

}

void CVUMetter::SetParentWindow(CClientDC &parent)
{
	pw=& parent;
}

void CVUMetter::DrawArrow(const int &dept)
{
	if(pw==NULL)return;
	if(dept==oldDept)return;
	if(dept<0)return;
	else if(dept>100)return;
	double angleR=(double)dept-50.0;
	double angleB=(double)oldDept-50.0;
	CPen	* OLDP,CPRED(PS_SOLID,2,RGB(255,50,50));
	CPen		   CPBLACK(PS_SOLID,2,RGB(20,40,20));
	CPen		   CPWHITE(PS_SOLID,1,RGB(200,200,200));
	OLDP=pw->SelectObject(&CPWHITE);
	int xr,xb,yr,yb;
	int Ax,Ay,cx,cy;
	Ax=(LARX)/2;
	Ay=(LARY*7)/8;
	cx=Ax+ox;
	cy=Ay+oy;
	for(double ag=-50;ag<=50;ag+=10)
	{
		CoSin(Ax,Ay,ag,xr,yr);
		CoSin(Ax-10,Ay-10,ag,xb,yb);
		pw->MoveTo(cx+xb,cy-yb);
		pw->LineTo(cx+xr,cy-yr);
	};
	pw->SelectObject(&CPBLACK);
	CoSin(Ax,Ay,angleR,xr,yr);
	CoSin(Ax,Ay,angleB,xb,yb);
	cx=Ax+=ox;
	cy=Ay+=(oy+2);
	pw->MoveTo(Ax,Ay);
	Ax+=xb;Ay-=yb;
	pw->LineTo(Ax,Ay);
	pw->SelectObject(&CPRED);
	pw->MoveTo(cx,cy);
	cx+=xr;cy-=yr;
	pw->LineTo(cx,cy);
	pw->SelectObject(OLDP);
	CBrush	* OLDB,CBBACK(RGB(100,50,40));
	OLDB=pw->SelectObject(&CBBACK);
	pw->Rectangle(ox+1,oy+LARY-1,ox+LARX-1,oy+LARY-1-(LARY/4));
	pw->SelectObject(OLDB);
	oldDept=dept;
	PeakLed(FALSE);
}

void CVUMetter::ResetVu()
{
	for(int ag=0;ag<100;ag++)
		DrawArrow(ag);
	PeakLed(TRUE);
	LvL=100;
}

void CVUMetter::SetCorner(const int &px, const int &py)
{
	ox=px;
	oy=py;
	sx=ox+LARX+5;
	DrawBox();
}

void CVUMetter::DrawBox()
{
	if(pw==NULL)return;
	CBrush	* OLDB,CBBLACK(RGB(20,40,20));
	OLDB=pw->SelectObject(&CBBLACK);
	pw->Draw3dRect(ox,oy,LARX,LARY,RGB(255,255,30),RGB(150,150,50));
	pw->Rectangle(ox+1,oy+1,ox+LARX-1,oy+LARY-1);
	pw->Draw3dRect(sx,oy,SPEX+1,LARY,RGB(100,255,100),RGB(10,255,10));
	pw->Rectangle(sx+1,oy+1,sx+SPEX,oy+LARY-1);
	pw->SelectObject(OLDB);
}

void CVUMetter::CoSin(const int &Rx,const int &Ry, double fase, int &dx, int &dy)
{
	double x=(double)Rx;
	double y=(double)Ry;
	fase*=kPI;
	x*=sin(fase);
	y*=cos(fase);
	dx=(int)x;
	dy=(int)y;
}

void CVUMetter::RedrawArrow()
{
	int deptZ=oldDept;
	oldDept=-1;
	DrawArrow(deptZ);
}

void CVUMetter::Cycle(float level)
{
	int aL=(int)(level*100);
	if(aL>100)aL=100;
	if(aL>LvL)LvL=aL;
	DrawArrow(LvL);
	if(LvL>0)if((LvL-=5)<0)LvL=0;
}

void CVUMetter::DrawSpectrum(const float * buffer)
{
	int px=sx+1,py=oy+LARY-1,cy;
	float sum;
	CPen	* OLDP,CPS(PS_SOLID,1,RGB(200,200,0));
	CPen	CPB(PS_SOLID,1,RGB(20,40,100));
	OLDP=pw->SelectObject(&CPB);
	cy=oy+1;px=sx+1;int idx=0;
	for(;idx<255;idx++,px++)
	{
		pw->MoveTo(px,cy);
		pw->LineTo(px,py-zarc[idx]);
		if(zarc[idx]>0)if((zarc[idx]-=10)<0)zarc[idx]=0;
	}
	py--;px=sx+1;
	pw->SelectObject(CPS);
	for(idx=0;idx<255;idx++,px++)
	{
		// media de 2 sucessivos...
		sum=*(buffer++);
		sum+=*(buffer++);
		sum*=SpectScale; // Media(n/2) e escalar(n*100)
		// Desenhar...
		pw->MoveTo(px,py);
		cy=(int)sum;
		if(cy>98)cy=98;
		if(zarc[idx]<cy)zarc[idx]=cy;
		pw->LineTo(px,py-cy);
	}
	pw->SelectObject(OLDP);
}

void CVUMetter::SetSpectScale(float &scale)
{
	SpectScale=scale;
}

void CVUMetter::PeakLed(const BOOL & ON)
{
	if(ON && !PEAK)PEAK=10;
	if(PEAK)PEAK--;
	CBrush	Led0Red(RGB(50,50,50));
	CBrush	Led1Red(RGB(255,20,20));
	CPen	* OLDP,LedBox(PS_SOLID,1,RGB(30,30,30));
	CBrush	* OLDB;
	OLDP=pw->SelectObject(&LedBox);
	OLDB=pw->SelectObject((PEAK)?&Led1Red:&Led0Red);
		int px=ox+LARX-30;
		int py=oy+LARY-(LARY/3);
		pw->Rectangle(px,py,px+15,py+6);
	pw->SelectObject(OLDP);
	pw->SelectObject(OLDB);

}
